package org.purl.jh.esslli;

import org.purl.jh.esslli.analysers.Analyser;
import org.purl.jh.esslli.analysers.Cascade;
import org.purl.jh.esslli.data.Lt;
import com.google.common.base.Charsets;
import com.google.common.base.Preconditions;
import com.google.common.io.Files;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Collection;

/**
 * The main class of the system to be called from the command line.
 */
public class Ma {
    private Analyser analyzer;

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        new Ma().go(args);
    }

    private void go(String[] args) {
        Preconditions.checkArgument(args.length == 3, "Expecting <lgSpecdir> <inFile> <outFile> " + args.length);
        
        File lgDir   = new File(args[0]);   // directory with morphology specification files
        File inFile  = new File(args[1]);   // file to analyze
        File outFile = new File(args[2]);   // result
        
        setupMorph(lgDir);
        
        try {
            analyze(inFile, outFile);
        } catch (IOException ex) {
            System.err.println(ex);
        }
    }

    private void setupMorph(File lgDir) {
        analyzer = new Cascade.Compiler().compile(lgDir);
    }

    /**
     * Analyzes all words in a file writing the results to another file.
     * @param inFile file to analyze, one word per line
     * @param outFile file to write results to
     */
    private void analyze(File inFile, File outFile) throws IOException {
        try (final PrintWriter w = new PrintWriter(Files.newWriter(outFile, Charsets.UTF_8))) {
            for (String form : Files.readLines(inFile, Charsets.UTF_8)) {
                Collection<Lt> lts = this.analyzer.analyze(form);
                write(w, form, lts);
            }        
        }
    }

    /**
     * Writes analysis of a single word to the supplied writer.
     * The point is just to write results to see them, in a real system we would 
     * pass the results to some configurable corpus writer.
     */
    private void write(PrintWriter w, String form, Collection<Lt> lts) {
        w.println(form);
        for (Lt lt : lts) {
            w.print("\t");
            w.print(lt.lemma);
            w.print("\t");
            w.println(lt.tag);
        }
    }
}
