package org.purl.jh.esslli.analysers;

import com.google.common.collect.ImmutableList;
import java.io.File;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import lombok.Data;
import org.purl.jh.esslli.data.Lt;

@Data
public class Cascade implements Analyser {

    private final List<Analyser> analysers;
    
    @Override
    public Collection<Lt> analyze(String form) {
        for (Analyser analyser : analysers) {
            Collection<Lt> lts = analyser.analyze(form);
            if (!lts.isEmpty()) return lts;
        }
        
        return Collections.emptySet();
    }

    
    public static class Compiler implements Analyser.Compiler {
        @Override
        public Cascade compile(File file) {
            Analyser a1 = new MapAnalyser.Compiler().compile(new File(file, "map.spec"));
            Analyser a2 = new MapAnalyser.Compiler().compile(new File(file, "modern.spec"));
            Analyser a3 = new GuesserCompiler().compile(new File(file, "endings.spec"));
            
            return new Cascade(ImmutableList.of(a1, a2, a3));
        }

    }
    
}
