package org.purl.jh.esslli.analysers;

import com.google.common.collect.Multimap;
import org.purl.jh.esslli.data.Lt;
import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import lombok.RequiredArgsConstructor;
import org.purl.jh.esslli.data.Ending;

@RequiredArgsConstructor
public class Guesser implements Analyser {
    private final int minStemLen;
    private final int maxEndingLen;
    
    private final Multimap<String,Ending> endings;
    
    @Override
    public Collection<Lt> analyze(String form) {
        final int n = form.length();
        final int from = Math.max(minStemLen, n-maxEndingLen);
        
        final List<Lt> result = new ArrayList<>();
        
        for(int split = from; split <= n; split++) {
            final String stemCand   = form.substring(0, split);
            final String endingCand = form.substring(split, n);

            for (Ending e : endings.get(endingCand)) {
                final Collection<String> lemmatizingStems = e.getStemRule().apply(stemCand);
                
                if (!lemmatizingStems.isEmpty()) {
                    for (String ls : lemmatizingStems) {
                        result.add(new Lt(ls + e.getLemmatizingEnding(), e.getTag()));
                    }
                }
            }
        }
        
        return result;
    }
}
