package org.purl.jh.esslli.analysers;

import com.google.common.base.Splitter;
import com.google.common.collect.ImmutableMultimap;
import com.google.common.collect.Iterables;
import com.google.common.collect.Multimap;
import java.io.File;
import java.util.List;
import org.purl.jh.esslli.data.Ending;
import org.purl.jh.esslli.data.Lt;
import org.purl.jh.esslli.data.StringRule;
import org.purl.jh.esslli.data.StringRuleUtil;
import org.purl.jh.esslli.data.TailChangeRule;
import org.purl.jh.esslli.util.IO;

public class GuesserCompiler implements Analyser.Compiler {
    @Override
    public Guesser compile(File file) {
        // todo load from file
        Multimap<String,Ending> endings = loadEndings(file  );

        int maxEndingLen = maxEndingLen(endings);

        return new Guesser(/*todo configurable*/ 1, maxEndingLen, endings);
    }

    private int maxEndingLen(Multimap<String, Ending> endings) {
        int max = 0;
        for (Ending e : endings.values()) {
            if (e.getStr().length() > max) max = e.getStr().length();
        }
        return max;
    }
    
    private Multimap<String,Ending> loadEndings(File file) {
        final ImmutableMultimap.Builder<String,Ending> builder = ImmutableMultimap.builder();

        final List<List<String>> splitLines = IO.readSplitStrings(file, 3);

        // format: endings lemmatizingEnding tags (@ rule)
        for (List<String> lineTokens : splitLines) {
            String endings = lineTokens.get(0);
            String lemmatizingEnding = lineTokens.get(1);

            List<String> tags;
            StringRule stemRule;
            if (Iterables.getLast(lineTokens).equals("@")) {
                tags = lineTokens.subList(2, lineTokens.size()-2);
                stemRule = TailChangeRule.fromSpecString(lineTokens.get(lineTokens.size()-2));
            }
            else {
                tags = lineTokens.subList(2, lineTokens.size());
                stemRule = StringRuleUtil.IDENTITY;
            }
            
            for (String endingStr : Splitter.on('/').split(endings)) {
                for (String tag : tags) {
                    builder.put(endingStr, new Ending(endingStr, tag, lemmatizingEnding, stemRule));
                }
                
            }
        }
        
        
        return builder.build();
    }
    
}
