package org.purl.jh.esslli.analysers;

import org.purl.jh.esslli.data.Lt;
import com.google.common.collect.ImmutableMultimap;
import com.google.common.collect.ImmutableSet;
import java.io.File;
import java.util.Collection;
import java.util.List;
import lombok.RequiredArgsConstructor;
import org.purl.jh.esslli.util.IO;

@RequiredArgsConstructor
public class MapAnalyser implements Analyser {
    private final com.google.common.collect.Multimap<String,Lt> dic;

    @Override
    public Collection<Lt> analyze(String form) {
        Collection<Lt> lts = dic.get(form);
        return lts == null ? ImmutableSet.<Lt>of() : lts;
    }
    
    public static class Compiler implements Analyser.Compiler {
        @Override
        public MapAnalyser compile(File file) {
            final List<List<String>> splitLines = IO.readSplitStrings(file, 1);
            
            final ImmutableMultimap.Builder<String,Lt> builder = ImmutableMultimap.builder();
            
            for (List<String> splitLine : splitLines) {
                String form = splitLine.get(0);

                if (splitLine.size() == 1) continue; 
                
                String lemma;
                List<String> tags;
                if (splitLine.get(1).equals("@")) {
                    lemma = splitLine.get(2);
                    tags = splitLine.subList(3, splitLine.size());
                }
                else {
                    lemma = form;
                    tags = splitLine.subList(1, splitLine.size());
                }
                
                for (String tag : tags) {
                    builder.put(form, new Lt(lemma,tag));
                }
            }
            
            return new MapAnalyser(builder.build());
        }
    }
    
    
}
