package org.purl.jh.esslli.data;

import com.google.common.collect.ImmutableSet;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import lombok.Data;

public class StringRuleUtil {

    public final static StringRule IDENTITY = new StringRule() {
        @Override
        public Set<String> apply(String input) {
            return ImmutableSet.of(input);
        }
    };

    @Data
    public static class Opt implements StringRule {
        private final StringRule rule;

        @Override
        public Set<String> apply(String input) {
            return ImmutableSet.<String>builder()
                .add(input)
                .addAll(rule.apply(input))
                .build();
        }
    }

    @Data
    public static class Sequence implements StringRule {
        private final List<StringRule> rules;

        @Override
        public Set<String> apply(String input) {
            return apply(input, rules);
        }

        private Set<String> apply(String string, Iterable<StringRule> rules) {
            Set<String> result = ImmutableSet.of(string);
            for (StringRule rule : rules) {
                result = apply(result, rule);
            }
            return result;
        }

        private Set<String> apply(Set<String> input, StringRule rule) {
            final Set<String> result = new HashSet<>();
            for (String stem : input) {
                result.addAll(rule.apply(stem));
            }
            return result;
        }
    }
    
}
